import sys
import subprocess


class NotifyHandler(object):

  def __init__(self, player):
    self.player = player
    self.isatty = sys.stdin.isatty()
    self.notify = self._notify_send(['-v'])

  def print_message(self, message):
    if message and self.isatty:
      sys.stdout.write("\x1b[2K\r%s" % message)
      sys.stdout.flush()

  def send_message(self, message):
    if message and self.notify and not self.isatty:
      name = 'Acestream Launcher'
      icon = self.player.split()[0]
      args = ['-h', 'int:transient:1', '-i', icon, name, message]

      self._notify_send(args)

  def print_stats(self, stream, stats):
    labels = { 'dl': 'playing', 'prebuf': 'buffering' }
    status = labels[stream.status]

    labels = 'down: %.1f kb/s up: %.1f kb/s peers: %d'
    sstats = labels % (stats.speed_down, stats.speed_up, stats.peers)

    if stream.is_live:
      label = 'LIVE - status: %s %s'
      stats = (status, sstats)
    else:
      label = 'VOD - status: %s total: %.2f%% %s'
      stats = (status, stats.total_progress, sstats)

    self.print_message(label % stats)

  def _notify_send(self, args):
    try:
      command = ['notify-send'] + args
      subprocess.call(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
      return True
    except OSError:
      return False



import os
import sys

try:
  import configparser
except ImportError:
  import ConfigParser as configparser


class ConfigHandler(object):

  def __init__(self):
    self.isatty = sys.stdin.isatty()
    self.config = configparser.RawConfigParser({
      'engine':  'acestreamengine --client-console',
      'host':    '127.0.0.1',
      'port':    '6878',
      'player':  'mpv',
      'timeout': '30',
      'verbose': 'False'
    })

    self._read_config()

  def get(self, option):
    return self._getoption(option, 'get')

  def getint(self, option):
    return self._getoption(option, 'getint')

  def getboolean(self, option):
    return self._getoption(option, 'getboolean')

  def _getoption(self, option, method):
    section = 'tty' if self.isatty else 'browser'
    return getattr(self.config, method)(section, option)

  def _read_config(self):
    self.config.add_section('tty')
    self.config.add_section('browser')
    self.config.read(os.path.expanduser('~/.config/acestream-launcher/config'))



import time
import argparse
import subprocess

from acestream_launcher.config import ConfigHandler
from acestream_launcher.notify import NotifyHandler

from acestream_launcher.stream import StreamHandler
from acestream_launcher.player import PlayerHandler


class StreamLauncher(object):

  stream  = None
  player  = None
  exiting = False

  def __init__(self):
    config = ConfigHandler()

    parser = argparse.ArgumentParser(
      prog='acestream-launcher',
      description='Open acestream links with any media player'
    )
    parser.add_argument(
      'url',
      metavar='URL',
      help='the acestream url to play'
    )
    parser.add_argument(
      '-e', '--engine',
      help='the engine command to use (default: acestreamengine --client-console)',
      default=config.get('engine')
    )
    parser.add_argument(
      '-p', '--player',
      help='the media player command to use (default: mpv)',
      default=config.get('player')
    )
    parser.add_argument(
      '-t', '--timeout',
      help='time in seconds to wait for stream playback (default: 30)',
      default=config.getint('timeout')
    )
    parser.add_argument(
      '-v', '--verbose',
      help='show engine and media player output in console',
      action='store_true',
      default=config.getboolean('verbose')
    )

    self.host = config.get('host')
    self.port = config.getint('port')
    self.args = parser.parse_args()
    self.noty = NotifyHandler(self.args.player)

  @property
  def output(self):
    output = None if self.args.verbose else subprocess.PIPE
    return { 'stdout': output, 'stderr': output }

  def notify(self, message):
    if message and not self.exiting:
      self.noty.print_message(message)
      self.noty.send_message(message)

  def stats(self, stream, stats):
    if not self.exiting:
      self.noty.print_stats(stream, stats)

  def play(self, url):
    self.player = PlayerHandler(bin=self.args.player)

    self.player.connect('notify', self.notify)
    self.player.connect('error', self.quit)
    self.player.connect('terminated', self.quit)

    self.player.start(url=url, **self.output)

  def run(self):
    self.stream = StreamHandler(
      bin=self.args.engine,
      host=self.host,
      port=self.port,
      timeout=self.args.timeout
    )

    self.stream.connect('notify', self.notify)
    self.stream.connect('stats::updated', self.stats)
    self.stream.connect('playing', self.play)
    self.stream.connect('error', self.quit)
    self.stream.connect('terminated', self.quit)

    self.stream.start(param=self.args.url, **self.output)

    while self.stream or self.player:
      time.sleep(1)

  def quit(self):
    if not self.exiting:
      self.exiting = True
      print('\n\nExiting...')

    if self.stream:
      self.stream.stop()
      self.stream = None

    if self.player:
      self.player.stop()
      self.player = None


def main():
  try:
    launcher = StreamLauncher()
    launcher.run()
  except KeyboardInterrupt:
    launcher.quit()


if __name__ == '__main__':
  main()



import os
import signal
import subprocess

from threading import Thread
from acestream.object import Observable


class PlayerHandler(Observable):

  def __init__(self, bin='mpv'):
    Observable.__init__(self)

    self.args    = bin.split()
    self.process = None

  def start(self, url, **kwargs):
    cmargs = self.args
    cmargs.append(url)

    thread = Thread(target=self._start_process, args=(cmargs), kwargs=kwargs)
    thread.setDaemon(True)
    thread.start()

  def stop(self):
    if bool(self.process):
      os.killpg(os.getpgid(self.process.pid), signal.SIGTERM)

      self.process = None
      self.emit('terminated')

  def _start_process(self, *args, **kwargs):
    try:
      self.process = subprocess.Popen(args, preexec_fn=os.setsid, **kwargs)
      self.emit('started')
      self.process.communicate()

      self.process = None
      self.emit('terminated')
    except OSError:
      self.process = None
      self.emit('notify', 'Media player not found in provided path!')
      self.emit('error')



import os
import re
import time

from acestream.object import Observable
from acestream.server import Server
from acestream.engine import Engine
from acestream.stream import Stream


class StreamHandler(Observable):

  def __init__(self, bin, host='127.0.0.1', port=6878, timeout=30):
    Observable.__init__(self)

    self.stream    = None
    self.params    = None
    self.playing   = False
    self.available = False
    self.timeout   = int(timeout)
    self.server    = Server(host=host, port=port)
    self.engine    = Engine(bin=bin)

  def start(self, param, **kwargs):
    self.params = self._parse_stream_param(param)

    self.emit('notify', 'Connecting to Acestream engine...')
    self._start_engine(**kwargs)

  def stop(self):
    self.engine.stop()

  def _start_engine(self, **kwargs):
    if not self.server.available:
      self.engine.connect('started', self._on_engine_started)
      self.engine.connect('terminated', self._on_engine_terminated)
      self.engine.connect('error', self._on_engine_error)

      self.engine.start(daemon=True, **kwargs)
    else:
      self._on_engine_started()

  def _start_stream(self):
    self.stream = Stream(self.server, **self.params)

    self.stream.connect('status::changed', self._on_stream_status_changed)
    self.stream.connect('stats::updated', self._on_stream_stats_updated)
    self.stream.connect('error', self._on_stream_error)

    self.stream.start()

  def _parse_stream_param(self, param):
    param = param.strip()
    files = re.search(r'\.(acelive|torrent)$', param)

    if files and not param.startswith('file://'):
      param = "file://%s" % os.path.realpath(param)

    if re.search(r'^(http|https|file)://', param):
      return { 'url': param }

    param = param.lower()
    match = re.search(r'[0-9a-f]{40}|[a-z2-7]{32}', param)

    if match and param.startswith('acestream://'):
      return { 'id': match[0] }

    if match:
      return { 'infohash': match[0] }

  def _timeout(self, object, attribute, message):
    while self.timeout > 0 and not getattr(object, attribute):
      self.timeout = self.timeout - 1
      time.sleep(1)

    if self.timeout == 0:
      self.emit('notify', message)
      self.emit('error')

  def _on_engine_started(self):
    self._timeout(self.server, 'available', 'Cannot connect to Acestream engine!')
    self.emit('notify', 'Waiting for stream response...')

    self._start_stream()
    self._timeout(self, 'available', 'Stream unavailable!')

  def _on_engine_terminated(self):
    self.emit('terminated')

  def _on_engine_error(self, _message=None):
    self.emit('notify', 'Acestream engine not found in provided path!')
    self.emit('error')

  def _on_stream_status_changed(self):
    if self.stream.status and not self.available:
      self.available = True

    if self.stream.status == 'dl' and not self.playing:
      self.playing = True

      self.emit('notify', 'Streaming started, launching player...')
      self.emit('playing', self.stream.playback_url)

  def _on_stream_stats_updated(self):
    self.emit('stats::updated', self.stream, self.stream.stats)

  def _on_stream_error(self, _message=None):
    self.emit('notify', 'Stream unavailable!')
    self.emit('error')



import setuptools

with open('README.md', 'r') as fh:
  long_description = fh.read()

setuptools.setup(
  name='acestream-launcher',
  version='2.0.3',
  author='Jonian Guveli',
  author_email='jonian@hardpixel.eu',
  description='Open AceStream links with a Media Player of your choice',
  long_description=long_description,
  long_description_content_type='text/markdown',
  url='https://github.com/jonian/acestream-launcher',
  packages=setuptools.find_packages(),
  data_files=[
    ('share/applications', ['acestream-launcher.desktop'])
  ],
  install_requires=[
    'acestream>=0.1.7'
  ],
  classifiers=[
    'Programming Language :: Python :: 2',
    'Programming Language :: Python :: 3',
    'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
    'Operating System :: POSIX :: Linux'
  ],
  project_urls={
    'Bug Reports': 'https://github.com/jonian/acestream-launcher/issues',
    'Source': 'https://github.com/jonian/acestream-launcher',
  },
  entry_points={
    'console_scripts': ['acestream-launcher = acestream_launcher.launcher:main']
  }
)
